/*
    File:       Animation.h

    Function:	

    Author:     Andrew Willmott

    Copyright:  (c) 2000, Andrew Willmott
*/

#ifndef __Animation__
#define __Animation__

#include "gcl/Avars.h"
#include "gcl/Hermite.h"
#include "gcl/Geometry.h"
#include "cl/Hash.h"

const GCLReal kAutoVel = -999999.0;
const GCLReal kAutoSplitVel = -999998.0;


class AnimRec
// keyframes for a single variable
{
public:
	AnimRec() : currentTimeIndex(-2) {};

	Void			SetupAnim();

	Void			AddKey(GCLReal time, GCLReal value, 
						GCLReal velIn, GCLReal velOut);

	GCLReal			GetValue(GCLReal time);

	Int				FindTimeIndex(GCLReal time);

	// fields
	ScalarList		keyTimes;
	ScalarList		values;
	ScalarList		velIn;
	ScalarList		velOut;

	Int				currentTimeIndex;
	Cubic			currentCubic;
};

typedef PtrArray<AnimRec> AnimRecList;

class Animation
// holds AnimRecs for all avars being tweaked.
{
public:
	Bool				ParseAnimFile(StrConst filename);

	Void				DumpAvars(GCLReal time, Int frame);
						///< print current avar values
	Void				SetAvarList(scAvarList *avarList, scCamera *itsCamera, GCLReal time);
						///< set scene's avar list according to time
	Void				SetupAnim();
						///< 

	Void				AddKey(StrConst avarName, GCLReal time, GCLReal value,
							GCLReal velIn = kAutoVel, GCLReal velOut = kAutoVel);
	GCLReal				GetValue(StrConst avarName, GCLReal time);

	IntHash				avarNames;
	AnimRecList			avarRecs;
	GCLReal				length;
	Int					quiet;
	Int					addDash;
	StrConst			tag;
	IndexList			avarMap;
};


#endif
