/*
	File:			Avars.h
	
	Function:		Provides avar-controlled scene objects, and methods
					for extracting a list of avars from the current scene.

	Author(s):		Andrew Willmott

	Copyright:		(c) 1995-2000, Andrew Willmott
 */

#ifndef __Avars__
#define __Avars__

#include "gcl/SceneObjects.h"
#include "cl/NArray.h"

// --- Avar class -------------------------------------------------------------

class Avar 
{
public:
	Avar()
	 {};
	Avar(
		StrConst	name,
		GCLReal		value 		= 0.0, 
		GCLReal		lowerBound 	= 0.0,
		GCLReal		upperBound 	= 1.0
	) :
		name(name), value(value), 
		lowerBound(lowerBound), upperBound(upperBound) {};

	StrConst	name;		// Avar's name
	GCLReal		value;		// its value
	GCLReal 	lowerBound;	// its range
	GCLReal 	upperBound;
};

ostream  &operator << (ostream &s, const Avar &avar);

typedef NArray<Avar> AvarList;

class scAvarList : public scAttribute, public AvarList
{
public:
	scAvarList() : scAttribute(aAvarList), AvarList() {};
	
	Void			Print(ostream &s) const;
	Object  	 	*Clone() const { return new scAvarList(SELF); };

	Bool			SetAvar(StrConst name, GCLReal value);
//	Void			AddAvar(StrConst name);

//	IntHash			avarNames;
};

class CreateAvarList: public scSceneAction
{
public:
	CreateAvarList() : scSceneAction(), avarList(0) {};
	
	Void		Start();
	Void		Attribute(scAttribute *sa);

	scAvarList	*avarList;
	Avar		newAvar;
};

// --- Avar-capable attributes -------------------------------------------------

class scAvarColour : public scColour
{
public:	
	scAvarColour(const Avar &avar, Colour colour);

	Bool 			HasAvar(Avar &avar, Int slot);
	Void			AddToContext(SLContext *context);
	Void			Print(ostream &s) const;
	Void			Parse(istream &s);
					
	virtual Void	Recalc();
	Object  		*Clone() const { return new scAvarColour(SELF); };
		
	Avar			avar;
	Colour			colour;
	Int				listSlot;
};

class scAvarEmittance : public scEmittance
{
public:	
	scAvarEmittance(const Avar &avar, Colour colour);

	Bool 			HasAvar(Avar &avar, Int slot);
	Void			AddToContext(SLContext *context);
	Void			Print(ostream &s) const;
	Void			Parse(istream &s);

	virtual Void	Recalc();
	Object  		*Clone() const { return new scAvarEmittance(SELF); };
		
	Avar			avar;
	Colour			colour;
	Int				listSlot;
};

class scAvarScalef : public scTransform
{
public:	
	scAvarScalef(const Avar &avar);

	Bool 			HasAvar(Avar &avar, Int slot);
	Void			AddToContext(SLContext *context);
	Void			Print(ostream &s) const;
	Void			Parse(istream &s);

	virtual Void	Recalc();
	Object  		*Clone() const { return new scAvarScalef(SELF); };
		
	Avar			avar;
	Int				listSlot;
};

class scAvarTransform : public scTransform
{
public: 
	scAvarTransform(const Avar &avar, Vector vector);

	Bool 			HasAvar(Avar &avar, Int slot);
	Void			AddToContext(SLContext *context);

	virtual Void	Recalc() {};
	Object  		*Clone() const { return new scAvarTransform(SELF); };
		
	Avar			avar;
	Vector			vector;
	Int				listSlot;
};

class scAvarShift : public scAvarTransform
{
public:	
	scAvarShift(const Avar &avar, Vector vector) : scAvarTransform(avar, vector) { Recalc();};
	Void			Recalc();
	Void			Print(ostream &s) const;
	Void			Parse(istream &s);
	Object  		*Clone() const { return new scAvarShift(SELF); };
};

class scAvarRotation : public scAvarTransform
{
public:	
	scAvarRotation(const Avar &avar, Vector vector) : scAvarTransform(avar, vector) { Recalc();};
	Void	Recalc();
	Void			Print(ostream &s) const;
	Void			Parse(istream &s);
	Object  		*Clone() const { return new scAvarRotation(SELF); };
};

class scAvarScale : public scAvarTransform
{
public:	
	scAvarScale(const Avar &avar, Vector vector) : scAvarTransform(avar, vector) { Recalc();};
	Void	Recalc();
	Void			Print(ostream &s) const;
	Void			Parse(istream &s);
	Object  		*Clone() const { return new scAvarScale(SELF); };
};

class scAvarSwitch : public scAttribute
{
public:	
	scAvarSwitch(const Avar &avar);

	Bool 			HasAvar(Avar &avar, Int slot);
	Void			AddToContext(SLContext *context);
	Void			Print(ostream &s) const;
	Void			Parse(istream &s);

	Object  		*Clone() const { return new scAvarSwitch(SELF); };
		
	Avar			avar;
	Int				listSlot;
};
 
#endif
