/*
	File:			EPSRenderer.h

	Function:		Renders line drawings to a .eps file.
	
	Author(s):		Andrew Willmott

	Copyright:		(c) 1997-2000, Andrew Willmott
 */

#ifndef __EPSRenderer__
#define __EPSRenderer__

#include "gcl/Renderer.h"
#include "gcl/EPSPlot.h"
#include "gcl/Clipper.h"

// --- The EPSRenderer class --------------------------------------------------


class EPSRenderer : public Renderer, public Clipper
{
public:
	EPSRenderer();

	Void				Attach(StrConst epsFilename);
	
	virtual Void		Show();
	virtual Void 		MakeCurrent();
	virtual Void 		Print(ostream &s);

	virtual Renderer	&Begin(RenderStyle style);
	virtual Renderer	&End();

	virtual Renderer	&SetPoint(const Point &p);
	virtual Renderer	&SetNormal(const Vector &p);
	virtual Renderer	&SetCoord(const Coord &c);
	virtual Renderer	&SetColour(const Colour &c);

	virtual Renderer	&SetTransform(const Transform &t);
	virtual Renderer	&SetCamera(const Camera &c);

	virtual Renderer	&Clear();
	virtual Renderer	&Pop();
	virtual Renderer	&Push();

	virtual Renderer	&GetImage(Image &image) // unsupported
						{return SELF;};
	virtual Renderer	&PutImage(const Image &image, Int x = 0, Int y = 0)
						{return SELF;};

	EPSPlot				itsPlot;

protected:
	Void				ViewPoint(const HPoint &hp, Bool draw);

	Array<Transform>	matrixStack;
	RenderStyle			style;
	Bool				start;
	Point				firstPoint;
	String				itsFilename;
};

#endif
