/*
	File:			Forms.cc

	Function:		See header file

	Author(s):		Andrew Willmott

	Copyright:		(c) 1997-2000, Andrew Willmott

	Notes:			

*/

// NOTE: not currently part of the GCL lib -- included separately
// by a few programs.

#include "gcl/Forms.h"

 
// --- Forms Graphics System --------------------------------------------------


FormsGraphicsSystem::FormsGraphicsSystem(int argc, char **argv, char *name) :
	XGraphicsSystem()
{
	itsEventsMask = StructureNotifyMask;	

	if (name == 0)
		name = argv[0];

	fl_initialize(&argc, argv, name, 0, 0);
}

int FormsXCallback(XEvent *event, Void *data)
{
	((XEventPane *) data)->XHandleEvent(event);
	
	return(1);
}

Void FormsGraphicsSystem::CreateFormsPane(XEventPane *xevpane, FL_OBJECT *object)
{
	const Int inset = 4;
	
	CreateSubwindow(xevpane, FL_OBJECT_WID(object), object->x + inset, 
		object->y + inset, object->w - 2 * inset, object->h - 2 * inset);
		
	fl_add_event_callback(xevpane->paneXID, Expose, FormsXCallback, xevpane);
	fl_add_event_callback(xevpane->paneXID, ButtonPress, FormsXCallback, xevpane);
	fl_add_event_callback(xevpane->paneXID, KeyPress, FormsXCallback, xevpane);

	fl_activate_event_callbacks(xevpane->paneXID);
}

Void FormsGraphicsSystem::CreateWindow(XEventPane *xevpane, Char *title, 
		Int width, Int height)
{
	XGraphicsSystem::CreateWindow(xevpane, title, width, height);
		
	fl_add_event_callback(xevpane->paneXID, Expose, FormsXCallback, xevpane);
	fl_add_event_callback(xevpane->paneXID, ButtonPress, FormsXCallback, xevpane);
	fl_add_event_callback(xevpane->paneXID, KeyPress, FormsXCallback, xevpane);

	fl_activate_event_callbacks(xevpane->paneXID);
}

Void FormsGraphicsSystem::GetMouse(XEventPane *pane, Int *x, Int *y,
								   UInt *keyState)
{
	fl_get_win_mouse(pane->paneXID, x, y, keyState);
}

Void FormsGraphicsSystem::Spin()
{
	FL_OBJECT *choice;
	
	choice = fl_check_forms(); 

	if (choice != 0)
		cerr << "Uncaptured forms event: " << choice << endl;
}

Void FormsGraphicsSystem::Run()
{
	finished = false;
	fl_do_forms();
	finished = true;
}

Void FormsGraphicsSystem::SignalDone()
{
	fl_finish();
}

// --- The Field Class --------------------------------------------------------


ostream &show(ostream &s)
{
	Field *cs = (Field *) &s;	// C++: hate, hate, hate...
								// There is no way to do this safely, i.e.,
	cs->Show();					//  guarantee it will only be run on a Field.

	return(s);
}
  
Void Field::Show()
{
// glibc under linux seems to have a bug whereby strstreams don't terminate
// strings returned by str() properly.
	Char glibcWorkaround[1024];

	memcpy(glibcWorkaround, str(), pcount());
	glibcWorkaround[pcount()] = 0;

	fl_set_object_label(object, glibcWorkaround);
	fl_check_only_forms();
}

Void InputField::Show()
{
	fl_set_input(object, str());
}


// --- Extra forms routines ---------------------------------------------------


Void fl_set_menu_item_check(FL_OBJECT *obj, int item, Bool b)
{
	int other = fl_get_menu_item_mode(obj, item) & 
		~(FL_PUP_CHECK | FL_PUP_BOX);

	if (!b)
		fl_set_menu_item_mode(obj, item,
			other | FL_PUP_BOX);
	else
		fl_set_menu_item_mode(obj, item,
			other | FL_PUP_CHECK);
}

Bool fl_get_menu_item_check(FL_OBJECT *obj, int item)
{
	return(fl_get_menu_item_mode(obj, item) & FL_PUP_CHECK);
}

Void fl_update_colors(FL_FORM *form)
{
	FL_OBJECT *obj;

	for (obj = form->first; obj != 0; obj = obj->next)
	{
		if (obj->active == 1 && obj->lcol == FL_INACTIVE)
		{
			fl_set_object_lcol(obj, FL_BLACK);
			fl_set_object_color(obj, obj->col1, FL_BLACK);
		}
		else if (obj->active == -1 && obj->lcol == FL_BLACK)
		{
			fl_set_object_lcol(obj, FL_INACTIVE);
			fl_set_object_color(obj, obj->col1, FL_INACTIVE);
		}
	}
}

Void fl_deactivate_group(FL_OBJECT *obj)
{
	fl_deactivate_object(obj);
	fl_update_colors(obj->form);
}

Void fl_activate_group(FL_OBJECT *obj)
{
	fl_activate_object(obj);
	fl_update_colors(obj->form);
}

Void my_activate(FL_OBJECT *obj)
{
	fl_activate_object(obj);
	fl_set_object_lcol(obj, FL_BLACK);
	fl_set_object_color(obj, obj->col1, FL_BLACK);
}

Void my_deactivate(FL_OBJECT *obj)
{
	fl_deactivate_object(obj);
	fl_set_object_lcol(obj, FL_INACTIVE);
	fl_set_object_color(obj, obj->col1, FL_INACTIVE);
}


// --- Form class -------------------------------------------------------------

Form::Form() : title("untitled")
{
}

Void ObjFormCallback(FL_OBJECT *object, Void *data)
{
	if (object->u_vdata)
		((Form *) object->u_vdata)->Event(object);
	else
		Expect(false, "Received event for unhandled object.");
}

Void Form::RegisterForm(FL_FORM *form)
{
	fl_set_form_callback(form, ObjFormCallback, 0);
}

Void Form::RegisterObj(FL_OBJECT *object)
{
	object->u_vdata = this;
}

Void Form::Event(FL_OBJECT *object)
{
	cerr << "Received event for object :" << object << " in form " << this 
		 << "." << endl;
}

Void Form::Show()
{
	fl_show_form(form, FL_PLACE_SIZE, FL_FULLBORDER, title);
}

Void Form::Hide()
{
	fl_hide_form(form);
}

Void Form::SetTitle(const Char *title)
{
	this->title = title;
}

