/*
	File:			Geometry.cc

	Function:		Implements Geometry.h

	Author(s):		Andrew Willmott

	Copyright:		(c) 1995-2000, Andrew Willmott

	Notes:			

*/

#include "gcl/Geometry.h"
#include "gcl/VecUtil.h"

#include <stdarg.h>

IndexList Indexes(Int first, ...)
{
	IndexList result;
	va_list ap;
	
	va_start(ap, first);
	
	while (first != IDX_END)	
	{
		result.Append(first);
		first = va_arg(ap, int);
	}
	
	va_end(ap);
	
	return(result);
}

Transform Align(const Vector &xAxis, const Vector &yAxis, const Vector &zAxis)
{
	Transform	align;

	// Create a matrix which will align scene with the xAxis, yAxis, zAxis
	align[0] = MVector4(xAxis[0], yAxis[0], zAxis[0], 0.0);
	align[1] = MVector4(xAxis[1], yAxis[1], zAxis[1], 0.0);
	align[2] = MVector4(xAxis[2], yAxis[2], zAxis[2], 0.0);
	align[3] = vl_w;
#ifdef VL_ROW_ORIENT
	align = trans(align);
#endif

	return(align);
}

Transform AlignToDir(const Vector &dir)
{
	Vector	x, y, z;
	
	if (sqrlen(dir) == 0.0)
		return(vl_I);
	y = norm(dir);
	x = norm(FindOrthoVector(y));
	z = cross(x, y);

	return(Align(x, y, z));
}


#ifdef CL_TMPL_INST
template class Array<Vector>;
#endif
