/*
    File:       Hermite.h

    Function:	Provides cubic interpolation functions

    Author:     Andrew Willmott

    Copyright:  (c) 1998-2000, Andrew Willmott
*/

#ifndef __Hermite__
#define __Hermite__

#include "gcl/Geometry.h"

/*
	Calculate hermite coefficients.

	t0, y0, ySlope0 are start time, value, and slope.
	t1, y1, ySlope1 are end time, value, and slope.
	t1 had better not equal t0.
		
	For automatic slope control, set 
		ySlope1 = (y2 - y0) / (t2 / t0);
		ySlope0 = (y1 - y-1) / (t1 / t-1);
*/

typedef Vector4	Cubic;

Void CalcHermiteCoeffs(
		GCLReal		t0,
		GCLReal		t1,
		GCLReal		y0,
		GCLReal		y1,
		GCLReal		ySlope0,
		GCLReal		ySlope1,
		Cubic		&result
	);

/*
	Evaluate y(t), using coeffs computed by CalcHermiteCoeffs().
	t0, t1 have to be same as you supplied to CalcHermiteCoeffs,
	and t should be between t0 and t1, inclusive.
*/

GCLReal EvalCubic(Cubic &cubicCoeffs, GCLReal t, GCLReal t0, GCLReal t1);

#endif
