/*
    File:       Quaternion.h

    Function:	Useful routines for using quaternions in CG

    Author:     Andrew Willmott

    Copyright:  (c) 2000, Andrew Willmott
*/

#ifndef __Quaternion__
#define __Quaternion__

#include "gcl/Geometry.h"

typedef Vector4 Quaternion;

Quaternion MakeQuat(const Vector &axis, GCLReal theta);
Quaternion MakeQuat(const Vector &point);
Quaternion MakeQuat(const VecTrans &t);
// R is assumed to contain the orthonormal axes of the new (rotated)
// space in its columns.
Quaternion QuatMult(const Quaternion &a, const Quaternion &b);

inline Quaternion QuatInv(const Quaternion &q)
{ return(Quaternion(-q[0], -q[1], -q[2], q[3])); }

inline Void MakeFromQuat(const Quaternion &q, VecTrans &t)
{ t.MakeRot(q); }

#endif
