/*
	File:			AnaRad.cc

	Function:		See header file

	Author(s):		Andrew Willmott

	Copyright:		(c) 1997-2000, Andrew Willmott

	Notes:			

*/

#include "AnaRad.h"
#include "AnaMesh.h"
#ifdef RAD_VIS
#include "gcl/Forms.h"
#endif

Bool AnaRad::Render()
//	Returns true if no interruptions.
{
	Int			i, j;
	Colour		patchIrr;
	GCLReal		save;
	GCLReal		sumArea, sumAF, sumAFLog, patchFactor;
	Point		p;
	Vector		n;

	RadMethod::Render();
	 
	lights.Clear();
	if (!gRadControl->patchView)
		for (j = 0; j < baseElems.NumItems(); j++)
			if (len(baseElems[j]->Emittance()) > 0)	// A light!
				lights.Append(baseElems[j]);
	
	gRadControl->jitterRot = false;
	sumArea = 0.0;
	sumAF = 0.0;
	sumAFLog = 0.0;
	theMI = 0.0;
		
	Stage(1);

	for (i = 0; i < patches.NumItems(); i++)
	{
		p = patches[i]->Centre();
		n = patches[i]->Normal();
		
		patchIrr = cBlack;
		sumArea += patches[i]->area;

		if (lights.NumItems() > 0)
		{
			for (j = 0; j < lights.NumItems(); j++)	
			{
				patchFactor = lights[j]->PatchFactor(p, n);
				if (patchFactor > 0)
					patchFactor *= lights[j]->Visibility(patches[i]);

				sumAF += patches[i]->area * patchFactor;
				sumAFLog += patches[i]->area * patchFactor * 
					log(patchFactor / lights[j]->area);

				patchIrr += patchFactor * lights[j]->Emittance();
			}
		}
		else
		{
			for (j = 0; j < patches.NumItems(); j++)	
			{
				patchFactor = patches[j]->PatchFactor(p, n);
				if (patchFactor > 0)
					patchFactor *= patches[j]->Visibility(patches[i]);

				sumAF += patches[i]->area * patchFactor;
				sumAFLog += patches[i]->area * patchFactor * 
					log(patchFactor / patches[j]->area);

				patchIrr += patchFactor * patches[j]->Emittance();
			}
		}	

		patches[i]->SetColour(patches[i]->Emittance() + 
			patches[i]->Reflectance() * patchIrr);
	}

	cout << sumAFLog << ", " << sumAF << ", " << sumArea << endl;
	theMI = (sumAFLog + sumAF * log(sumArea)) / sumArea;
	
	Stage(2);
	
	return(0);
}

RadElem *AnaRad::NewMesh()
{
	return(new AnaGrid);
}

#ifdef RAD_VIS

Void AnaRad::DumpStats(){}

Int AnaRad::Stage(Int stage)
{
	switch (stage)
	{
	case 1:
		break;
	case 2:
		ColourVertices();
		display->Redraw();
		Field(out1) << "Mutual Information: " << theMI << show;
		Field(out2) << "illum patches: " << lights.NumItems()
					<< " patches:" << patches.NumItems() << show;
		Field(out3) << "Rays: " << gRadControl->rays << show;
		break;
	}

	if (Pause()) return(1);
	
	return(0);
}

#else

Void AnaRad::DumpStats()
{
	cout << dumpID
		<< ' ' << totTime
		<< ' ' << gRadControl->stage
		<< ' ' << lights.NumItems()
		<< ' ' << patches.NumItems()
		<< ' ' << gRadControl->rays
		<< ' ' << theMI
		<< endl;

	DumpScene();
}

Int AnaRad::Stage(Int stage)
{
	if (CheckTime()) return(1);

	gRadControl->stage = stage;

	switch (stage)
	{
	case 1:		// pre setup
		cout << "method ana " << endl;
		cout << "sub " << gRadControl->patchSubdivs << endl;
		cout << "scene " << sceneName << endl;
		cout << "format ID time stage illumPatches patches rays MI" << endl; 
		cout << "-------------------------------------------------" << endl;
	
		DumpStats();
		break;

	case 2:
		DumpStats();
		break;
				
	default:
		break;
	}

	if (Idle()) return(0);
	timer.ContTimer();	
	return(0);
}

#endif

