/*
	File:		Flatlets.h

	Function:	Implements flatlet bases F2 and F3

	Author:		Andrew Willmott

    Copyright:  (c) 1996-2000, Andrew Willmott
*/

#ifndef __Flatlets__
#define __Flatlets__
 
#include "HRMesh.h"


// --- F2 Flatlets ------------------------------------------------------------


class F2Elem;

class F2Link : public HRLink
{
public:
	HRLink			*New() { return new F2Link; };

	F2Elem			*To()	{ return((F2Elem*) to); };
	F2Elem			*From()	{ return((F2Elem*) from); };

	Void			Gather();
	Bool			CalcTransport();
	Bool			CalcVisibility(HRLink *parent, Bool forceVisReuse);
	GCLReal			Error();
	GCLReal			BFAError();

	Void			Print(ostream &s);
	Void			DrawLink(Renderer &r, GCLReal left, GCLReal top, 
							 GCLReal right, GCLReal bottom, GCLReal weight);

	// Fields	
	GCLReal			transport[16];		// wavelength independent.
	GCLReal			factor;
};

class F2Elem : public HRMeshElem
{
public:
	Void			SetParent(HierElem &parent);
	StrConst		Name() { return("f2"); };
	HierElem		*New() { return(new F2Elem); };

	Colour			SampleLeaf(Coord c);
	Void			Print(ostream &s);
	Void			PrintSelf(ostream &s);
	Void			ParseSelf(istream &s);

	Void			DistributeColours();
	
	Void			Add();
	Void			Push();
	Void			Pull();
	Void			CalcLeafRadiosity();
	GCLReal			Error();
	Void			ClearB();
	Void			ClearR();

	Void			DrawLeaf(Renderer &r);
	Int				Coeffs() { return(4); };

	// Generic wavelet interface
	Void			EltSampleTransport(Int numSamples, Point p[], Vector n[],
								   GCLMat &coeffs);
	GCLReal			EltCalcTransport(HRElem *from, GCLMat&coeffs);
	Void			AddIrradiance(const Colour &E, const Vector &m);
	Colour			GetPower(const Vector &m);

	Int				NumCoeffs() { return(4); };
	Colour			*B_Coeffs() { return(B); };
	Colour			*R_Coeffs() { return(R); };

	// Fields
	Colour			B[4];
	Colour			R[4];
	static Colour	lastB[4];
};


// --- F3 Flatlets ------------------------------------------------------------


class F3Elem;

class F3Link : public HRLink
{
public:
	HRLink			*New() { return new F3Link; };

	F3Elem			*To()	{ return((F3Elem*) to); };
	F3Elem			*From()	{ return((F3Elem*) from); };

	Void			Gather();
	Bool			CalcTransport();
	Bool			CalcVisibility(HRLink *parent, Bool forceVisReuse);
	GCLReal			Error();
	GCLReal			BFAError();

	Void			Print(ostream &s);
	Void			DrawLink(Renderer &r, GCLReal left, GCLReal top,
							 GCLReal right, GCLReal bottom, GCLReal weight);

	GCLReal			transport[81];
	GCLReal			factor;
};

class F3Elem : public HRMeshElem
{
public:
	Void			SetParent(HierElem &parent);
	StrConst		Name() { return("f3"); };
	HierElem		*New() { return(new F3Elem); };
	
	Colour			SampleLeaf(Coord c);
	Void			Print(ostream &s);
	Void			PrintSelf(ostream &s);
	Void			ParseSelf(istream &s);

	Void			DistributeColours();
	
	Void			Add();
	Void			Push();
	Void			Pull();
	Void			CalcLeafRadiosity();
	GCLReal			Error();
	Void			ClearB();
	Void			ClearR();

	Void			DrawLeaf(Renderer &r);
	Int				Coeffs() { return(9); };

	// Generic wavelet interface
	Void			EltSampleTransport(Int numSamples, Point p[], Vector n[],
								   GCLMat &coeffs);
	GCLReal			EltCalcTransport(HRElem *from, GCLMat&coeffs);
	Void			AddIrradiance(const Colour &E, const Vector &m);
	Colour			GetPower(const Vector &m);

	Int				NumCoeffs() { return(9); };
	Colour			*B_Coeffs() { return(B); };
	Colour			*R_Coeffs() { return(R); };

	// Fields
	Colour			B[9];
	Colour			R[9];
	static Colour	lastB[9];
};

#endif
