/*
	File:			VLc.h

	Function:		Master header for a version of the VL library based on complex numbers.
					The various classes are named Vecc, Mat3c, SparseVecc, etc.
					
	Author(s):		Andrew Willmott

	Copyright:		(c) 1995-2000, Andrew Willmott
 */


#ifndef __VLf__
#define __VLf__

#ifdef __VL__
#include "vl/VLUndef.h"
#endif

#include "cl/Basics.h"
#include <complex.h>

inline Real len(float_complex c)
{ return(sqrt(sqr(real(c)) + sqr(imag(c)))); }
inline Real sqrlen(float_complex c)
{ return(sqr(real(c)) + sqr(imag(c))); }
inline Void SetReal(float_complex &a, Real b)
{ a = b; }

#define VL_V_REAL float_complex
#define VL_V_SUFF(X) X ## c
#define VL_M_REAL float_complex
#define VL_M_SUFF(X) X ## c

#include "vl/Vec2.h"
#include "vl/Vec3.h"
#include "vl/Vec4.h"
#include "vl/Vec.h"
#include "vl/SparseVec.h"

#include "vl/Mat2.h"
#include "vl/Mat3.h"
#include "vl/Mat4.h"
#include "vl/Mat.h"
#include "vl/SparseMat.h"
#include "vl/Solve.h"
#include "vl/Transform.h"

#endif
